/**
 * @prop {String} value - Value selected for the date picker.
 * @prop {String} valueEnd - Value selected for the second date picker when using date range.
 * @attr {String} error - When defined, sets persistent validity to `customError` and sets `setCustomValidity` = attribute value.
 * @attr {String} validity - Specifies the `validityState` this element is in.
 * @attr {String} setCustomValidity - Sets a custom help text message to display for all validityStates.
 * @attr {String} setCustomValidityRangeUnderflow - Custom help text message to display when validity = `rangeUnderflow`.
 * @attr {String} setCustomValidityRangeOverflow - Custom help text message to display when validity = `rangeOverflow`.
 * @attr {String} setCustomValidityValueMissing - Help text message to display when validity = `valueMissing`.
 * @attr {String} calendarStartDate - The first date that may be displayed in the calendar.
 * @attr {String} calendarEndDate - The last date that may be displayed in the calendar
 * @attr {String} calendarFocusDate - The date that will first be visually rendered to the user in the calendar.
 * @attr {Boolean} disabled - If set, disables the datepicker.
 * @attr {Boolean} noValidate - If set, disables auto-validation on blur.
 * @attr {Boolean} required - Populates the `required` attribute on the input. Used for client-side validation.
 * @attr {Boolean} range - If set, turns on date range functionality in auro-calendar.
 * @attr {String} centralDate - The date that determines the currently visible month.
 * @attr {String} maxDate - Maximum date. All dates after will be disabled.
 * @attr {String} minDate - Minimum date. All dates before will be disabled.
 * @attr {Array} monthNames = Names of all 12 months to render in the calendar, used for localization of date string in mobile layout.
 * @slot helpText - Defines the content of the helpText.
 * @slot mobileDateLabel - Defines the content to display above selected dates in the mobile layout.
 * @slot toLabel - Defines the label content for the second input when the `range` attribute is used.
 * @slot fromLabel - Defines the label content for the first input.
 * @slot date_MM_DD_YYYY - Defines the content to display in the auro-calendar-cell for the specified date. The content text is colored using the success state token when the `highlight` attribute is applied to the slot.
 * @slot popover_MM_DD_YYYY - Defines the content to display in the auro-calendar-cell popover for the specified date.
 * @csspart dropdown - Use for customizing the style of the dropdown.
 * @csspart trigger - Use for customizing the style of the datepicker trigger.
 * @csspart input - Use for customizing the style of the datepicker inputs.
 * @csspart calendarWrapper - Use for customizing the style of the calendar container.
 * @csspart calendar - Use for customizing the style of the calendar.
 * @csspart helpTextSpan - Use for customizing the style of the datepicker help text span.
 * @csspart helpText - Use for customizing the style of the datepicker help text.
 * @event auroDatePicker-ready - Notifies that the component has finished initializing.
 * @event auroDatePicker-valueSet - Notifies that the component has a new value set.
 * @event auroDatePicker-toggled - Notifies that the calendar dropdown has been opened/closed.
 * @event auroDatePicker-monthChanged - Notifies that the visible calendar month(s) have changed.
 * @event auroFormElement-validated - Notifies that the component value(s) have been validated.
 * @event auroDatePicker-newSlotContent - Notifies that new slot content has been added to the datepicker.
 */
export class AuroDatePicker extends LitElement {
    static get properties(): {
        error: {
            type: StringConstructor;
            reflect: boolean;
        };
        noValidate: {
            type: BooleanConstructor;
        };
        setCustomValidity: {
            type: StringConstructor;
            reflect: boolean;
        };
        setCustomValidityRangeUnderflow: {
            type: StringConstructor;
            reflect: boolean;
        };
        setCustomValidityRangeOverflow: {
            type: StringConstructor;
            reflect: boolean;
        };
        setCustomValidityValueMissing: {
            type: StringConstructor;
            reflect: boolean;
        };
        validity: {
            type: StringConstructor;
            reflect: boolean;
        };
        range: {
            type: BooleanConstructor;
            reflect: boolean;
        };
        disabled: {
            type: BooleanConstructor;
            reflect: boolean;
        };
        required: {
            type: BooleanConstructor;
            reflect: boolean;
        };
        type: {
            type: StringConstructor;
            reflect: boolean;
        };
        value: {
            type: StringConstructor;
            reflect: boolean;
        };
        valueEnd: {
            type: StringConstructor;
            reflect: boolean;
        };
        centralDate: {
            type: StringConstructor;
        };
        maxDate: {
            type: StringConstructor;
            reflect: boolean;
        };
        minDate: {
            type: StringConstructor;
            reflect: boolean;
        };
        monthNames: {
            type: ArrayConstructor;
        };
        calendarStartDate: {
            type: StringConstructor;
            reflect: boolean;
        };
        calendarEndDate: {
            type: StringConstructor;
            reflect: boolean;
        };
        calendarFocusDate: {
            type: StringConstructor;
            reflect: boolean;
        };
        /**
         * @private
         */
        dropdownElementName: {
            type: StringConstructor;
        };
        /**
         * @private
         */
        dropdownTag: {
            type: ObjectConstructor;
        };
        /**
         * @private
         */
        inputElementName: {
            type: StringConstructor;
        };
        /**
         * @private
         */
        inputTag: {
            type: ObjectConstructor;
        };
    };
    static get styles(): import("lit").CSSResult[];
    /**
     * This will register this element with the browser.
     * @param {string} [name="auro-datepicker"] - The name of element that you want to register to.
     *
     * @example
     * AuroDatePicker.register("custom-datepicker") // this will register this element to <custom-datepicker/>
     *
     */
    static register(name?: string): void;
    /**
     * @private
     */
    private util;
    /**
     * @private
     */
    private calendarRenderUtil;
    disabled: boolean;
    required: boolean;
    range: boolean;
    noValidate: boolean;
    validity: any;
    value: any;
    valueEnd: any;
    calendarStartDate: any;
    calendarEndDate: any;
    calendarFocusDate: any;
    monthNames: string[];
    /**
     * @private
     */
    private type;
    /**
     * @private
     */
    private dateSlotContent;
    /**
     * @private
     */
    private validation;
    /**
     * @private
     */
    private runtimeUtils;
    /**
     * @private
     */
    private forceScrollOnNextMobileCalendarRender;
    dropdownTag: any;
    inputTag: any;
    /**
     * Force the calendar view to the focus date when it changes.
     * @private
     * @returns {void}
     */
    private handleFocusDateChange;
    /**
     * @private
     * @param {Number} length - Number of characters for the returned random string.
     * @returns {string}
     */
    private generateRandomString;
    /**
     * Focuses the datepicker trigger input.
     * @param {String} focusInput - Pass in `endDate` to focus on the return input. No parameter is needed to focus on the depart input.
     * @returns {void}
     */
    focus(focusInput: string): void;
    /**
     * Converts valid time number to format used by wc-date-range API.
     * @private
     * @param {Date} date - Date to be converted.
     * @returns {Number} Simplified number.
     */
    private convertToWcValidTime;
    /**
     * Converts date object into a string.
     * @private
     * @param {String} time - Unix timestamp to be converted to a date object.
     * @returns {Date} Date formatted as a string.
     */
    private convertWcTimeToDate;
    /**
     * Sends event notifying that the input has changed it's value.
     * @private
     * @returns {void}
     */
    private notifyValueChanged;
    /**
     * Generates a date string used in the mobile calendar layout.
     * @private
     * @param {string} date - Date to parse into longer mobile version.
     * @returns {string}
     */
    private getMobileDateStr;
    /**
     * Return appropriate error message.
     * @param {Object} evt - Event passed in from auro-input when the event triggered this function.
     * @private
     */
    private getErrorMessage;
    errorMessage: any;
    /**
     * Changes the calendar's visibility to reflect the value of the central date attribute.
     * @private
     * @returns {void}
     */
    private handleCentralDateChange;
    /**
     * Sends event notifying that the calendar popover has been opened.
     * @private
     * @returns {void}
     */
    private notifyDatepickerToggled;
    /**
     * Sends event notifying that the calendar's visible month has changed.
     * @param {Object} event - Event passed in from auro-calendar when the event triggered this function.
     * @private
     * @returns {void}
     */
    private notifyMonthChanged;
    /**
     * Binds all behavior needed to the dropdown after rendering.
     * @private
     * @returns {void}
     */
    private configureDropdown;
    dropdown: any;
    /**
     * Binds all behavior needed to the input after rendering.
     * @private
     * @returns {void}
     */
    private configureInput;
    triggerInput: any;
    inputList: any[];
    setCustomValidity: any;
    /**
     * Binds all behavior needed to the dropdown after rendering.
     * @private
     * @returns {void}
     */
    private configureCalendar;
    calendar: Element;
    /**
     * Binds all behavior needed to the datepicker after rendering.
     * @private
     * @returns {void}
     */
    private configureDatepicker;
    /**
     * Marks the component as ready and sends event.
     * @private
     * @returns {void}
     */
    private notifyReady;
    ready: boolean;
    /**
     * Sets the readonly attribute on the inputs based on the window width.
     * @private
     * @returns {void}
     */
    private handleReadOnly;
    /**
     * Keep the datepicker in sync with the calendar's central date.
     * @private
     * @param {Number} event - Event received from calendar with the new central date.
     * @returns {void}
     */
    private handleCalendarCentralDateChange;
    /**
     * Sets the datepicker's values to the auro-calendar-cell that was clicked.
     * @private
     * @param {Number} time - Unix timestamp to be converted to a date.
     * @returns {void}
     */
    private handleCellClick;
    cellClickActive: boolean;
    /**
     * Emits an event to notify the calendar cells to fetch their slot content.
     * @returns {void}
     */
    pushSlotContent(): void;
    updated(changedProperties: any): void;
    firstUpdated(): void;
    render(): import("lit-html").TemplateResult;
}
import { LitElement } from "lit";
//# sourceMappingURL=auro-datepicker.d.ts.map